part of schema_transform;

class KeyCodec extends Codec<mirrorfree.Key, schema.Key> {

  final KeyEncoder _encoder;
  final KeyDecoder _decoder;
  KeyCodec(Map<String,mirrorfree.KindDefinition> _kindDefinitions):
    this._encoder = new KeyEncoder(),
    this._decoder = new KeyDecoder(_kindDefinitions);

  @override
  Converter<schema.Key, mirrorfree.Key> get decoder => _decoder;


  @override
  Converter<mirrorfree.Key, schema.Key> get encoder => _encoder;
}

class KeyEncoder extends Converter<mirrorfree.Key, schema.Key> {


  @override
  schema.Key convert(mirrorfree.Key input) {
    return new schema.Key()
      ..pathElement.addAll(input.path.map(_keyToPathElement));
  }

  schema.Key_PathElement _keyToPathElement(mirrorfree.Key key) {
    var pathElement = new schema.Key_PathElement()
        ..kind = key.kind.name;
    if (key.id != null)
      pathElement.id = new Int64(key.id);
    if (key.name != null)
      pathElement.name = key.name;
    return pathElement;

  }
}

class KeyDecoder extends Converter<schema.Key, mirrorfree.Key> {

  KindRegistry<String> _kindRegistry;

  KeyDecoder(this._kindRegistry);

  @override
  mirrorfree.Key convert(schema.Key input) {
    return input.pathElement.fold(null, _fromPathElement);
  }

  mirrorfree.Key _fromPathElement(mirrorfree.Key parent, schema.Key_PathElement pathElement) {
    var kind = _kindRegistry[pathElement.kind];
    var id = (pathElement.hasId()) ? pathElement.id.toInt() : null;
    var name = (pathElement.hasName()) ? pathElement.name : null;
    return new mirrorfree.Key(kind, parentKey: parent, id: id, name: name);
  }
}
