library schema_transform.key_test;

import 'package:unittest/unittest.dart';

import 'package:fixnum/fixnum.dart';

import 'package:google_cloud_datastore/src/schema_transform/schema_transform.dart';
import 'package:google_cloud_datastore/src/common.dart' as mirrorfree;
import 'package:google_cloud_datastore/src/proto/schema_v1_pb2.dart' as schema;


void main() {
  group('key', () {
    var kindReg = new KindRegistry({
        'Kind': new mirrorfree.KindDefinition('Kind', [], entityFactory: (key) => new mirrorfree.Entity(key))
    });
    var keyCodec = new KeyCodec(kindReg);
    test('should be able to convert a key into a schema key', () {
      var key = new mirrorfree.Key(
          kindReg['Kind'],
          name: 'emily',
          parentKey: new mirrorfree.Key(kindReg['Kind'], id: 123)
      );
      var encoded = keyCodec.encode(key);
      var pe1 = encoded.pathElement[0];
      expect(pe1.kind, 'Kind');
      expect(pe1.id, new Int64(123));
      expect(pe1.hasName(), isFalse);

      var pe2 = encoded.pathElement[1];
      expect(pe2.kind, 'Kind');
      expect(pe2.name, 'emily');
      expect(pe2.hasId(), isFalse);
    });

    test("should be able to convert a schema key into a key", () {
      var pe = [
          new schema.Key_PathElement()..kind = 'Kind'..id=new Int64(123),
          new schema.Key_PathElement()..kind = 'Kind'..name='emily'
      ];
      var schemaKey = new schema.Key()..pathElement.addAll(pe);
      var decoded = keyCodec.decode(schemaKey);

      expect(decoded.kind.name, 'Kind');
      expect(decoded.name, 'emily');
      expect(decoded.id, null);

      expect(decoded.parentKey.kind.name, 'Kind');
      expect(decoded.parentKey.name, null);
      expect(decoded.parentKey.id, 123);
    });
  });
}